/* 

   part of the shansyn spherical harmonics package, see COPYRIGHT for license 

   $Id: read_she_model.c,v 1.9 2001/07/14 19:51:26 becker Exp $ 

*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h>
#include "function_macros.h"
#include "trig_constants.h"
#include "precision.h"
#include "spherical_harmonics_functions.h"
#include "legendre_macros.h"
#include "myio.h"


/*
  read in spherical harmonic expansion model with depth layers
  
  the frist integer, the layer number n, determines the type of 
  model. 

  0<n<1000: discrete layer, will need to read depths
  n < 0   : Chebyshev expansion of order -n
  1000<=n : splines interpolation of order n-1000
  
  fname: filename
  lmax: 
         if the input value lmax is set to <=0, no change in l_max
         if > 0, the model resolution will be changed to l_max,
         adding zeroes if necessary
*/
void read_she_model(char *fname,struct mod *model,
		    int lmax)
{
  FILE *in;
  int j,lms,l,m;
  model->dmin=FLT_MAX;model->dmax=-FLT_MAX;
  in=fopen(fname,"r");
  if(!in){
    fprintf(stderr,"read_she_model: error opening spherical harmonics model file \"%s\"\n",
	    fname);
    exit(-1);
  }
  if(fscanf(in,"%i",&model->n)!=1){
    fprintf(stderr,"read_she_model: read error model file %s, could not read nl\n",fname);
    exit(-1);
  }
  if(model->n<0){
    model->n *= -1;
    model->type=CHEBYSHEV;
  }else{
    if(model->n>1000){
      model->type=SPLINES;
      model->n-=1000.0;
    }else
      model->type=DISCRETE;
  }
  if(model->n<1){
    fprintf(stderr,"read_she_model: read error model file %s, number of layers is %i\n",
	    fname,model->n);
    exit(-1);
  }
  model->lmax=0;
  if(!(model->d=(COMP_PRECISION *)
       malloc(sizeof(COMP_PRECISION)*model->n)))
    MEMERROR;
  for(j=0;j<model->n;j++){
    if(model->type == DISCRETE){// for discrete model, need to read in depth 
      if(fscanf(in,DATA_FSCAN_FORMAT,&model->d[j])!=1){
	fprintf(stderr,"read_she_model: read error model file %s, layer %i, could not read d, nl=%i, lmax=%i\n",
		fname,j,model->n,model->lmax);exit(-1);}
      if(model->d[j]<0){
	fprintf(stderr,"read_she_model: depth argument has to be positive\n");
	exit(-1);
      }
      if(j && (model->d[j]>model->d[j-1])){
	fprintf(stderr,
		"read_she_model: model %s\nread_she_model: discrete layers have to sorted be from bottom to top,\nread_she_model: largest z (positive) first, %i: %g %i: %g\n",
		fname,j,model->d[j],j-1,model->d[j-1]);
	exit(-1);
      }
    }else{// Chebyshev or splines model, read dmin and dmax only once
      if(j==0)
	if(fscanf(in,TWO_DATA_FSCAN_FORMAT,&model->dmin,
		  &model->dmax)!=2){
	  fprintf(stderr,"read_she_model: read error dmin / dmax Chebyshev model\n");
	}
    }
    if(model->type == DISCRETE){
      if(model->d[j] > model->dmax)
	model->dmax = model->d[j];
      if(model->d[j] < model->dmin)
	model->dmin = model->d[j];
    }
    if(fscanf(in,"%i",&model->lmax)!=1){
      fprintf(stderr,"read_she_model: read error file %s, layer %i could not read lmax\n",
	      fname,j);exit(-1);}
    if(j==0){// first layer, allocate pointers for coefficients
      lms= (int)((((COMP_PRECISION)model->lmax)+1.0)*(((COMP_PRECISION)model->lmax)+2)/2.0);
      if(lms==0){
	fprintf(stderr,"read_she_model: model %s, lms is %i\n",
		fname,lms);
	exit(-1);
      }
      model->a=(COMP_PRECISION **)malloc(sizeof(COMP_PRECISION *)*model->n);
      model->b=(COMP_PRECISION **)malloc(sizeof(COMP_PRECISION *)*model->n);
      if(!model->a || !model->b)MEMERROR;
    }
    model->a[j]=(COMP_PRECISION *)malloc(sizeof(COMP_PRECISION)*lms*model->n);
    model->b[j]=(COMP_PRECISION *)malloc(sizeof(COMP_PRECISION)*lms*model->n);
    if(!model->a[j] || !model->b[j])MEMERROR;
    for(l=0;l<=model->lmax;l++){
      for(m=0;m<=l;m++){
	if((fscanf(in,TWO_DATA_FSCAN_FORMAT,(model->a[j]+POSLM(l, m)),(model->b[j]+POSLM(l, m))))!=2){
	  fprintf(stderr,"read_she_model: read error file %s, layer %i, l %i, m %i\n",
		  fname,j,l,m);exit(-1);
	}
      }
    }
  }
  // modify l_max
  if(lmax > 0){
    lms= (int)((((COMP_PRECISION)lmax)+1.0)*(((COMP_PRECISION)lmax)+2)/2.0);
    for(j=0;j<model->n;j++){
      model->a[j]=(COMP_PRECISION *)
	realloc(model->a[j],sizeof(COMP_PRECISION)*lms*model->n);
      model->b[j]=(COMP_PRECISION *)
	realloc(model->b[j],sizeof(COMP_PRECISION)*lms*model->n);
      if(!model->a[j] || !model->b[j])MEMERROR;
      if(lmax>model->lmax)
	for(l=model->lmax+1;l<=lmax;l++)
	  for(m=0;m<=l;m++)
	    *(model->a[j]+POSLM(l, m))= *(model->b[j]+POSLM(l, m))=0.0;
      if(j==0)
	fprintf(stderr,"read_she_model: changed l_max from %i to %i\n",model->lmax,lmax);
    }
    model->lmax=lmax;
  }
  fclose(in);
  switch(model->type){
  case DISCRETE:{
    fprintf(stderr,"read_she_model: read in discrete layer model %s with %i layers, lmax is %i, dmin/max : %g %g\n",
	    fname,model->n,model->lmax,model->dmin,model->dmax);
    break;
  }
  case CHEBYSHEV:{
    fprintf(stderr,"read_she_model: read in Chebyshev model %s with order %i, lmax is %i, dmin/max : %g %g\n",
	    fname,model->n,model->lmax,model->dmin,model->dmax);
    break;
  }
  case SPLINES:{
    fprintf(stderr,"read_she_model: read in spline model %s with order %i, lmax is %i, dmin/max : %g %g\n",
	    fname,model->n,model->lmax,model->dmin,model->dmax);
    break;

  }
  default:{
    fprintf(stderr,"read_she_model: ended up with wrong model code: %i\n",
	    model->type);
    exit(-1);
  }}

    
}

